import java.awt.*;
import java.awt.image.*;
import java.util.*;

class Main {
    Image image;

    Main(String filename) {
        ImageInfoGrabber iig = new ImageInfoGrabber();

        image = Toolkit.getDefaultToolkit().getImage(filename);
        if (!iig.grabInfo(image.getSource())) {
            System.err.println("Error fetching image " + filename);
            System.exit(1);
        } 
        System.out.println("size = " + iig.width + "x" + iig.height);
        System.out.print("hints = ");
        if ((iig.hints & ImageConsumer.COMPLETESCANLINES) != 0) 
            System.out.print("COMPLETESCANLINES ");
        if ((iig.hints & ImageConsumer.RANDOMPIXELORDER) != 0) 
            System.out.print("RANDOMPIXELORDER ");
        if ((iig.hints & ImageConsumer.SINGLEPASS) != 0) 
            System.out.print("SINGLEPASS ");
        if ((iig.hints & ImageConsumer.TOPDOWNLEFTRIGHT) != 0) 
            System.out.print("TOPDOWNLEFTRIGHT ");
        System.out.println();
    
        System.out.println(iig.properties.size() + " properties");

        // Only deal with index color model images
        if (iig.colorModel instanceof IndexColorModel) {
            IndexColorModel cm = (IndexColorModel)iig.colorModel;
            int[] pixelValueCount = new int[1<<cm.getPixelSize()];
            System.out.println("transparent pixel = " 
                + cm.getTransparentPixel());

            // Count number of times pixel values are used.
            for (int i=0; i<iig.bytePixels.length; i++) {
                pixelValueCount[iig.bytePixels[i]&0xff]++;
            }

            // Print colors in color model and their reference count.
            for (int i=0; i<pixelValueCount.length; i++) {
                System.out.println(i 
                    + ": " + (cm.getRed(i)&0xff) 
                    + " " + (cm.getGreen(i)&0xff) 
                    + " " + (cm.getBlue(i)&0xff) 
                    + "/" + (cm.getAlpha(i)&0xff) 
                    + "   count:" + pixelValueCount[i]);
            }
        }
        System.exit(0);
    }

    static public void main(String[] args) {
        if (args.length != 1) {
            System.err.println("usage: java Main <image file>");
            System.exit(1);
        }
        new Main(args[0]);
    }
}

class ImageInfoGrabber implements ImageConsumer {
    // These are the public fields which the client can
    // use to retrieve the image info.
    public int width;
    public int height;
    public int hints;
    public Hashtable properties;
    public ColorModel colorModel;
    public Vector additionalColorModels = new Vector();

    // One of the following fields is null and the other is not.
    public int[] intPixels;
    public byte[] bytePixels;

    // Private fields
    private int status;
    private ImageProducer producer;

    // Returns true if the image was fetched successfully; false otherwise.
    public synchronized boolean grabInfo(ImageProducer ip) {
        status = 0;
        (producer = ip).startProduction(this);
        try {
            while (status == 0) {
                wait();
            }
        } catch (InterruptedException e) {
            return false;
        }
        return status > 0;
    }

    public void setDimensions(int w, int h) {
        width = w;
        height = h;
    }

    public void setHints(int h) {
        hints = h;
    }

    public void setProperties(Hashtable props) {
        properties = props;
    }

    public void setColorModel(ColorModel cm) {
        colorModel = cm;
    }

    public void setPixels(int srcX, int srcY, int srcW, int srcH,
                          ColorModel cm, byte pixels[], int srcOff, int srcScan) {
        if (cm != colorModel) {
            if (!additionalColorModels.contains(cm)) {
                additionalColorModels.addElement(cm);
            }
        }
        if (bytePixels == null) {
            bytePixels = new byte[width * height];
        }
        for (int x=srcX; x<srcX + srcW; x++) {
            for (int y=srcY; y<srcY + srcH; y++) {
                bytePixels[y * width + x] = 
                    pixels[(y-srcY) * srcScan + (x-srcX) + srcOff];
            }
        }
    }

    public void setPixels(int srcX, int srcY, int srcW, int srcH,
                          ColorModel cm, int pixels[], int srcOff, int srcScan) {
        if (cm != colorModel) {
            if (!additionalColorModels.contains(cm)) {
                additionalColorModels.addElement(cm);
            }
        }
        if (intPixels == null) {
            intPixels = new int[width * height];
        }
        for (int x=srcX; x<srcX + srcW; x++) {
            for (int y=srcY; y<srcY + srcH; y++) {
                intPixels[y * width + x] = 
                    pixels[(y-srcY) * srcScan + (x-srcX) + srcOff];
            }
        }
    }

    public synchronized void imageComplete(int s) {
        switch (s) {
        case STATICIMAGEDONE:
        case SINGLEFRAMEDONE:
            status = 1;
            break;
        default:
        case IMAGEERROR:
        case IMAGEABORTED:
            status = -1;
            break;
        }
        producer.removeConsumer(this);
        notifyAll();
    }
}
